@@ -1,8 +1,12 @@
 /*
- $Id: lfe.c 1671 2015-03-23 13:04:42Z sgaure $
+ $Id: lfe.c 1748 2015-07-10 20:29:00Z sgaure $
 */
 #include "lfe.h"
-
+SEXP MY_threads(SEXP rt) {
+  if(LENGTH(rt) < 1) return R_NilValue;
+  LFE_GLOBAL_THREADS = INTEGER(rt)[0];
+  return R_NilValue;
+}
 static R_CallMethodDef callMethods[] = {
   {"conncomp", (DL_FUNC) &MY_conncomp, 1},
   {"wwcomp", (DL_FUNC) &MY_wwcomp, 1},
@@ -16,7 +20,8 @@ static R_CallMethodDef callMethods[] = {
   {"dsyrk", (DL_FUNC) &MY_dsyrk, 4},
   {"dsyr2k", (DL_FUNC) &MY_dsyr2k, 5},
   {"address", (DL_FUNC) &MY_address, 1},
-
+  {"ppf", (DL_FUNC) &MY_ppf, 2},
+  //  {"threads", (DL_FUNC) &MY_threads, 1},
   {NULL, NULL, 0}
 };
 
@@ -24,6 +29,7 @@ void attribute_visible R_init_lfe(DllInfo *info) {
   /* register our routines */
   (void) R_registerRoutines(info,NULL,callMethods,NULL,NULL);
   (void) R_PreserveObject(df_string=mkString("data.frame"));
+  LFE_GLOBAL_THREADS=1;
 }
 
 void attribute_visible R_unload_lfe(DllInfo *info) {
